> module RevPowerlist where
> import Prelude hiding (succ)
> import Rank2 (Functor2, Mu2, In, it, Nu2, Coit, out)

The constructor of pure kind 2 which we need to define powerlists.  We
take the freedom and use labelled sums.

> data PListF f a = Zero a | Succ (f (a, a))

We give a monotonicity witness which performs reversal.

> swap :: (a -> b) -> (a,a) -> (b,b)
> swap f (a1, a2) = (f a2, f a1)
>
> pListFRev :: (forall a b. (a -> b) -> f a -> g b) ->
>              (a -> b) -> PListF f a -> PListF g b
>
> pListFRev s f (Zero a) = Zero (f a)
> pListFRev s f (Succ l) = Succ (s (swap f) l)

> instance Functor2 PListF where
>     ffmap s f l = pListFRev s f l

Inductive type of powerlists.

> type PList a = Mu2 PListF a

Powerlist constructors.

> zero :: a -> PList a
> zero a = In (Zero a)
>
> succ :: PList (a,a) -> PList a
> succ l = In (Succ l)

Fast powerlist reversal.

This is just the map function.

> pListRev :: PList a -> PList a
> pListRev = fmap id

\comm{ begin comment

Convert to list.

> unzipMap :: (a -> b) -> [(a,a)] -> [b]
> unzipMap f [] = []
> unzipMap f ((a1,a2):as) = f a1 : f a2 : unzipMap f as

> stepList :: PListF [ ] a -> [a]
> stepList (Zero a) = [a]
> stepList (Succ l) = unzipMap id l

> toList :: PList a -> [a]
> toList l = it stepList id l 

> showPList :: (a -> [b]) -> PList a -> [b]
> showPList f (In (Zero a)) = f a
> showPList f (In (Succ l)) = showPList (\ (a1,a2) -> f a1 ++ f a2) l

> toLDirect :: PList a -> [a]
> toLDirect l = showPList (\a -> [a]) l

Test.

> pl1  = succ (succ (succ (zero (((1, 2), (3, 4)), ((5, 6), (7,8))))))
> spl1 = toList pl1
> rl1  = pListRev pl1
> srl1 = toList rl1

> spl1d = toLDirect pl1
> srl1d = toLDirect rl1

end comment }